// This is a scripted coppercube action. It starts/stops dragging a 3d object with the mouse. 
// Add this action into the "when clicked onto this do something" behavior to make it dragable.
//
// The following embedded xml is for the editor and describes how the behavior can be edited:
// Supported types are: int, float, string, bool, color, vect3d, scenenode, texture, action
/*
	<action jsname="action_Drag3DObject" description="Drag 3D Object">
	</action>
*/

action_Drag3DObject = function()
{
};

currentlyDraggingAction = null;
dontRestartDraggingUntil = 0;

// called when the action is executed 
action_Drag3DObject.prototype.execute = function(currentNode)
{
	if (dontRestartDraggingUntil != 0)
	{
		var now = (new Date()).getTime();
		if (dontRestartDraggingUntil > now)
			return;
	}
	
	if (currentlyDraggingAction)
	{
		currentlyDraggingAction.stopDragging();
		return;
	}
			
	currentlyDraggingAction = this;
	
	var me = this; 
	this.registeredFunction = function() { me.draggingFunc(); }; 
	
	ccbRegisterOnFrameEvent(this.registeredFunction);	
	
	this.currentNode = currentNode;
	
	var positionNode = ccbGetSceneNodeProperty(currentNode, "Position");
	var positionCam =  ccbGetSceneNodeProperty(ccbGetActiveCamera(), "Position");	
	
	this.distance = positionNode.substract(positionCam).getLength();
}


action_Drag3DObject.prototype.stopDragging = function()
{
	this.currentNode = null;
	ccbUnregisterOnFrameEvent(this.registeredFunction);
	currentlyDraggingAction = null;
	
	dontRestartDraggingUntil = (new Date()).getTime() + 300;
}


action_Drag3DObject.prototype.draggingFunc = function()
{
	// update position
	
	var mouseX = ccbGetMousePosX();
	var mouseY = ccbGetMousePosY();

	var pos3d = ccbGet3DPosFrom2DPos(mouseX, mouseY);
	var positionCam =  ccbGetSceneNodeProperty(ccbGetActiveCamera(), "Position");	
	
	var ray = pos3d.substract(positionCam);
	ray.normalize();
	ray.x *= this.distance;
	ray.y *= this.distance;
	ray.z *= this.distance;
	
	ccbSetSceneNodeProperty(this.currentNode, "Position", ray.x + positionCam.x, ray.y + positionCam.y, ray.z + positionCam.z);
}
